package be.iminds.ilabt.jfed.restapi.resource;

import be.iminds.ilabt.jfed.fedmon.util.Clock;
import be.iminds.ilabt.jfed.fedmon.webapi.service.json.Log;
import be.iminds.ilabt.jfed.fedmon.webapi.service.json.LogBuilder;
import be.iminds.ilabt.jfed.restapi.dao.DebugDataDao;
import be.iminds.ilabt.jfed.restapi.JFedRestApiConfiguration;
import be.iminds.ilabt.jfed.restapi.debugdata.DebugDataManager;
import be.iminds.ilabt.jfed.restapi.debugdata.ExperimentDD;
import be.iminds.ilabt.jfed.util.GeniUrn;
import com.codahale.metrics.annotation.Timed;
import org.apache.commons.codec.binary.Base64;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.slf4j.MDC;

import javax.servlet.http.HttpServletRequest;
import javax.validation.constraints.NotNull;
import javax.ws.rs.*;
import javax.ws.rs.core.*;
import java.io.IOException;
import java.io.OutputStream;
import java.net.URI;
import java.net.URISyntaxException;
import java.nio.charset.Charset;

@Path("/debugdata/experiment")
@Produces(MediaType.APPLICATION_JSON)
public class DebugDataResource {
    private static final Logger LOG = LoggerFactory.getLogger(DebugDataResource.class);

    @Context
    UriInfo uriInfo;

    private final DebugDataManager debugDataManager;
    private final DebugDataDao debugDataDAO;
    private final JFedRestApiConfiguration configuration;

    public DebugDataResource(DebugDataManager debugDataManager, DebugDataDao debugDataDAO, JFedRestApiConfiguration configuration) {
        this.debugDataManager = debugDataManager;
        this.debugDataDAO = debugDataDAO;
        this.configuration = configuration;
    }

    @GET
    @Path("{id}")
    @Timed
    public ExperimentDD getExperimentDD(@NotNull @PathParam("id") Long id, @Context HttpServletRequest request) {
        GeniUrn authenticatedUser = configuration.getAuthenticatedUser(request);
        if (authenticatedUser == null) {
            throw new WebApplicationException("The SSL connection did not correctly authenticate you as a trusted user. Did you use SSL client authentication?",
                    Response.Status.FORBIDDEN);
        }

        //remove any leftover experimentId from MDC
        MDC.remove("experimentId");

        ExperimentDD res = debugDataManager.getExperimentDD(id);
        if (res == null)
            throw new NotFoundException("There is no experiment debug data with id="+id);

        if (res.getUserUrn() == null) {
            LOG.warn("ExperimentDD "+id+" has userUrn == null. Denying access to "+authenticatedUser);
            if (!configuration.hasAccess(JFedRestApiConfiguration.Access.ADMIN, request))
                throw new WebApplicationException("There is no user info in requested ExperimentDD -> you are denied access.",
                        Response.Status.FORBIDDEN);
        } else {
            if (!res.getUserUrn().equals(authenticatedUser) && !configuration.hasAccess(JFedRestApiConfiguration.Access.ADMIN, request)) {
                LOG.info("Denying user "+authenticatedUser+" access to ExperimentDD "+id+" belonging to "+res.getUserUrn());
                throw new WebApplicationException("Access denied: you may only access DebugData for your own Experiment",
                        Response.Status.FORBIDDEN);
            }
        }

        return res;
    }
}
